'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var convert = require('@analys/convert');
var matrixTranspose = require('@vect/matrix-transpose');
var iconv = _interopDefault(require('iconv-lite'));
var enumChars = require('@spare/enum-chars');

const popBlank = rows => {
  let {
    length: l
  } = rows,
      tail;

  while (l) {
    tail = rows[--l];
    if (tail.length && tail.some(x => x === null || x === void 0 ? void 0 : x.trim().length)) break;
    rows.pop();
  }

  return rows;
};

const parseCsvMap = (csvText, de = enumChars.CO, lf = enumChars.LF) => {
  return csvText.split(lf).map(row => row.split(de).map(word => word.trim()));
};

const csvReg = (de, qt) => {
  de = '\\' + de;
  const qt2 = qt + qt;
  const delimiter = `${de}|\\r?\\n|\\r|^`;
  const quotedFill = `[^${qt}]*(?:${qt2}[^${qt}]*)*`;
  const standardContent = `[^${qt}${de}\\r\\n]*`;
  return new RegExp(`(${delimiter})(?:${qt}(${quotedFill})${qt}|(${standardContent}))`, 'gi');
}; // const regexCsv = /(,|\x?\n|\x|^)(?:"([^"]*(?:""[^"]*)*)"|([^",\x\n]*))/gi

// e.g. dqRg = /""/g
const dualQuoteReg = qt => new RegExp(qt + qt, 'g');

const parseCsvReg = (text, de = ',', qt = '"') => {
  const reg = csvReg(de, qt),
        dualQuote = dualQuoteReg(qt),
        rows = [];
  let ms,
      sep,
      qtv,
      stv,
      row = [];

  while ((ms = reg.exec(text)) && ([, sep, qtv, stv] = ms)) {
    var _stv$trim, _stv;

    if (sep && sep !== de) rows.push(row), row = []; // if separator is line-feed, push new row.

    row.push(qtv ? qtv.replace(dualQuote, qt) : (_stv$trim = (_stv = stv) === null || _stv === void 0 ? void 0 : _stv.trim()) !== null && _stv$trim !== void 0 ? _stv$trim : '');
  } // if the captured value is quoted, unescape double quotes, else push the non-quoted value


  if (row.length) rows.push(row);
  return rows;
}; // reg,   // regexp to parse the CSV values
// ditto, // regexp for double quotes
// ms,    // array to hold individual pattern matching groups.
// sep,   // captured separator, can be either delimiter or line-feed.
// qtv,   // captured quoted value.
// stv,   // captured unquoted value.
// wd,    // final processed capture value.
// row,   // data row.

class NaiveCsv {
  /**
   *
   * @param {string|*} csvText
   * @param {string} [de=',']
   * @param {string} [lf='\x\n']
   * @param {string?} [qt]
   * @param {string?} [decode]
   * @param {boolean} [transpose]
   * @param {boolean} [popBlank]
   * @returns {*[][]}
   */
  static toRows(csvText, {
    de = ',',
    lf = '\r\n',
    // lf not applicable if qt is specified
    qt = '\"',
    decode,
    transpose,
    popBlank: popBlank$1
  } = {}) {
    var _ref, _mx, _mx2, _mx3;

    if (decode) csvText = iconv.decode(csvText, decode);
    let mx = qt ? parseCsvReg(csvText, de, qt) : parseCsvMap(csvText, de, lf);
    if (transpose) mx = popBlank$1 ? (_ref = (_mx = mx, popBlank(_mx)), matrixTranspose.transpose(_ref)) : (_mx2 = mx, matrixTranspose.transpose(_mx2));
    return popBlank$1 ? (_mx3 = mx, popBlank(_mx3)) : mx;
  }
  /**
   *
   * @param {string|*} csvText
   * @param {string} [de=',']
   * @param {string} [lf='\x\n']
   * @param {string?} [qt]
   * @param {string?} [decode]
   * @param {boolean} [transpose]
   * @param {boolean} [popBlank]
   * @param {string} [title]
   * @returns {{head, rows, title}}
   */


  static toTable(csvText, {
    de = ',',
    lf = '\r\n',
    qt,
    decode,
    transpose,
    popBlank,
    title
  } = {}) {
    const rows = NaiveCsv.toRows(csvText, {
      de,
      lf,
      qt,
      decode,
      transpose,
      popBlank
    }),
          head = rows.shift();
    return {
      head,
      rows,
      title
    };
  }
  /**
   *
   * @param {string|*} csvText
   * @param {string} [de=',']
   * @param {string} [lf='\x\n']
   * @param {string?} [qt]
   * @param {string?} [decode]
   * @param {boolean} [transpose]
   * @param {boolean} [popBlank]
   * @param {string} [title]
   * @returns {Object[]}
   */


  static toSamples(csvText, {
    de = ',',
    lf = '\r\n',
    qt,
    transpose,
    decode,
    popBlank
  } = {}) {
    const rows = NaiveCsv.toRows(csvText, {
      de,
      lf,
      qt,
      decode,
      transpose,
      popBlank
    }),
          head = rows.shift();
    return convert.tableToSamples({
      head,
      rows
    });
  }

}

exports.NaiveCsv = NaiveCsv;
